<?php

/**
 * @file
 * Implementations of emfield hooks for the Media: Captions module.
 */

/**
 * Implements hook_emfield_widget_extra().
 *
 * This is called by _emfield_emfield_widget (in emfield.cck.inc) when
 * building the widget on the node form. It creates a textarea element
 * so the editor can upload captions.
 */
function _media_captions_emfield_widget_extra($form, $form_state, $field, $items, $delta = 0, $module) {
  $element = array();
  if ($field['widget']['media_captions']) {
    $label = $field['widget']['media_captions_label'];
    $description = $field['widget']['media_captions_description'];
    $default_value = isset($items[$delta]['data']['media_captions']) ? $items[$delta]['data']['media_captions']['raw'] : '';
    $element['media_captions_section'] = array(
      '#type' => 'fieldset',
      '#title' => t('@label', array('@label' => $label)),
      '#collapsed' => FALSE,
      '#collapsible' => TRUE,
    );
    $element['media_captions_section']['media_captions'] = array(
      '#type' => 'textarea',
      '#description' => filter_xss($description),
      '#default_value' => $default_value,
    );
  }
  if ($field['widget']['media_captions_ad']) {
    $element['media_captions_ad'] = array(
      '#type' => 'media_captions_ad_widget',
      '#field_name' => $field['field_name'],
      '#delta' => $delta,
      '#type_name' => $form['type']['#value'],
      '#default_value' => $item,
      '#upload_validators' => media_captions_ad_widget_upload_validators($field),
    );
  }
  return $element;
}

/**
 * Get the upload validators for a file field.
 *
 * @param $field
 *   A CCK field array.
 * @return
 *   An array suitable for passing to file_save_upload() or the file field
 *   element's '#upload_validators' property.
 */
function media_captions_ad_widget_upload_validators($field) {
  $max_filesize = parse_size(file_upload_max_size());
  if (!empty($field['widget']['max_filesize_per_file']) && parse_size($field['widget']['max_filesize_per_file']) < $max_filesize) {
    $max_filesize = parse_size($field['widget']['max_filesize_per_file']);
  }

  // Match the default value if no file extensions have been saved at all.
  if (!isset($field['widget']['file_extensions'])) {
    $field['widget']['file_extensions'] = 'mp3';
  }

  $validators = array(
    // associate the field to the file on validation.
    'media_captions_validate_associate_field' => array($field),
    'media_captions_validate_size' => array($max_filesize),
    // Override core since it excludes uid 1 on the extension check.
    // Filefield only excuses uid 1 of quota requirements.
    'media_captions_validate_extensions' => array($field['widget']['file_extensions']),
  );
  return $validators;
}

/**
 * An #upload_validators callback. Check the file matches an allowed extension.
 *
 * If the mimedetect module is available, this will also validate that the
 * content of the file matches the extension. User #1 is included in this check.
 *
 * @param $file
 *   A Drupal file object.
 * @param $extensions
 *   A string with a space separated list of allowed extensions.
 * @return
 *   An array of any errors cause by this file if it failed validation.
 */
function media_captions_validate_extensions($file, $extensions) {
  global $user;
  $errors = array();

  if (!empty($extensions)) {
    $regex = '/\.('. ereg_replace(' +', '|', preg_quote($extensions)) .')$/i';
    $matches = array();
    if (preg_match($regex, $file->filename, $matches)) {
      $extension = $matches[1];
      // If the extension validates, check that the mimetype matches.
      if (module_exists('mimedetect')) {
        $type = mimedetect_mime($file);
        if ($type != $file->filemime) {
          $errors[] = t('The file contents (@type) do not match its extension (@extension).', array('@type' => $type, '@extension' => $extension));
        }
      }
    }
    else {
      $errors[] = t('Only files with the following extensions are allowed: %files-allowed.', array('%files-allowed' => $extensions));
    }
  }

  return $errors;
}

/**
 * Help text automatically appended to fields that have extension validation.
 */
function media_captions_validate_extensions_help($extensions) {
  if (!empty($extensions)) {
    return t('Allowed Extensions: %ext', array('%ext' => $extensions));
  }
  else {
    return '';
  }
}

/**
 * An #upload_validators callback. Check the file size does not exceed a limit.
 *
 * @param $file
 *   A Drupal file object.
 * @param $file_limit
 *   An integer value limiting the maximum file size in bytes.
 * @param $file_limit
 *   An integer value limiting the maximum size in bytes a user can upload on
 *   the entire site.
 * @return
 *   An array of any errors cause by this file if it failed validation.
 */
function media_captions_validate_size($file, $file_limit = 0, $user_limit = 0) {
  global $user;

  $errors = array();

  if ($file_limit && $file->filesize > $file_limit) {
    $errors[] = t('The file is %filesize exceeding the maximum file size of %maxsize.', array('%filesize' => format_size($file->filesize), '%maxsize' => format_size($file_limit)));
  }

  // Bypass user limits for uid  = 1.
  if ($user->uid != 1) {
    $total_size = file_space_used($user->uid) + $file->filesize;
    if ($user_limit && $total_size > $user_limit) {
      $errors[] = t('The file is %filesize which would exceed your disk quota of %quota.', array('%filesize' => format_size($file->filesize), '%quota' => format_size($user_limit)));
    }
  }
  return $errors;
}

/**
 * Automatic help text appended to fields that have file size validation.
 */
function media_captions_validate_size_help($size) {
  return t('Maximum Filesize: %size', array('%size' => format_size(parse_size($size))));
}

/**
 * An element #process callback for the media_captions_widget field type.
 *
 * Expands the media_captions_widget type to include the upload field, upload
 * and remove buttons, and the description field.
 */
function media_captions_ad_widget_element_process($element, $edit, &$form_state, $form) {
  // The widget is being presented, so apply the JavaScript.
//   drupal_add_js(drupal_get_path('module', 'media_captions') .'/media_captions.widget.js');
//   $element['#theme'] = 'media_captions_widget_item';

  $item = $element['#value'];
  $field_name = $element['#field_name'];
  $field = $form['#field_info'][$field_name];
  $delta = $element['#delta'];
  $label = $field['widget']['media_captions_ad_label'];
  $description = $field['widget']['media_captions_ad_description'];
  $default_value = isset($items[$delta]['data']['media_captions_ad']) ? $items[$delta]['data']['media_captions_ad']['fid'] : 0;

  $element['media_captions_ad_section'] = array(
    '#type' => 'fieldset',
    '#title' => t('@label', array('@label' => $label)),
    '#collapsed' => FALSE,
    '#collapsible' => TRUE,
  );

  // Set up the buttons first since we need to check if they were clicked.
  $element['media_captions_ad_section']['media_captions_upload'] = array(
    '#type' => 'submit',
    '#value' => t('Upload'),
    '#submit' => array('node_form_submit_build_node'),
    '#ahah' => array( // with JavaScript
       'path' => 'media-captions/emfield/ad/ahah/'.   $element['#type_name'] .'/'. $element['#field_name'] .'/'. $element['#delta'],
       'wrapper' => $element['#id'] .'-ahah-wrapper',
       'method' => 'replace',
       'effect' => 'fade',
    ),
    '#field_name' => $element['#field_name'],
    '#delta' => $element['#delta'],
    '#type_name' => $element['#type_name'],
    '#upload_validators' => $element['#upload_validators'],
    '#weight' => 100,
    '#post' => $element['#post'],
  );
  $element['media_captions_ad_section']['media_captions_remove'] = array(
    // With default CCK edit forms, $element['#parents'] is array($element['#field_name'], $element['#delta']).
    // However, if some module (for example, flexifield) places our widget deeper in the tree, we want to
    // use that information in constructing the button name.
    '#name' => implode('_', $element['#parents']) .'_media_captions_remove',
    '#type' => 'submit',
    '#value' => t('Remove'),
    '#submit' => array('node_form_submit_build_node'),
    '#ahah' => array( // with JavaScript
      'path' => 'media_captions/ahah/'.   $element['#type_name'] .'/'. $element['#field_name'] .'/'. $element['#delta'],
      'wrapper' => $element['#id'] .'-ahah-wrapper',
      'method' => 'replace',
      'effect' => 'fade',
    ),
    '#field_name' => $element['#field_name'],
    '#delta' => $element['#delta'],
    '#weight' => 101,
    '#post' => $element['#post'],
  );

  // Because the output of this field changes depending on the button clicked,
  // we need to ask FAPI immediately if the remove button was clicked.
  // It's not good that we call this private function, but
  // $form_state['clicked_button'] is only available after this #process
  // callback is finished.
  if (_form_button_was_clicked($element['media_captions_remove'])) {
    // Delete the file if it is currently unused. Note that media_captions_delete_file()
    // does a reference check in addition to our basic status check.
    if (isset($edit['fid'])) {
      $removed_file = media_captions_load_file($edit['fid']);
      if ($removed_file['status'] == 0) {
        media_captions_delete_file($removed_file);
      }
    }
    $item = array('fid' => 0);
  }

  // Set access on the buttons.
  $element['media_captions_ad_section']['media_captions_upload']['#access'] = empty($item['fid']);
  $element['media_captions_ad_section']['media_captions_remove']['#access'] = !empty($item['fid']);

    // Set the FID.
  $element['fid'] = array(
    '#type' => 'hidden',
    '#value' => $item['fid'],
  );

  if ($item['fid'] != 0) {
    $element['preview'] = array(
      '#type' => 'markup',
      '#value' => theme('media_captions_widget_preview', $item),
    );
  }

  $desc = array(filter_xss($description));
  foreach ($element['#upload_validators'] as $callback => $arguments) {
    $help_func = $callback .'_help';
    if (function_exists($help_func)) {
      $desc[] = call_user_func_array($help_func, $arguments);
    }
  }

  $element['media_captions_ad_section']['media_captions_ad'] = array(
    '#name' => 'files['. $element['#field_name'] .'_'. $element['#delta'] .']',
    '#type' => 'file',
    '#description' => implode('<br />', $desc),
    '#size' => 22,
    '#attributes' => array(
      'accept' => implode(',', array_filter(explode(' ', $field['widget']['file_extensions']))),
    ),
    '#access' => empty($item['fid']),
  );

  // Grant access to temporary files.
  if ($item['fid'] && isset($item['status']) && $item['status'] == 0 && variable_get('file_downloads', FILE_DOWNLOADS_PUBLIC) == FILE_DOWNLOADS_PRIVATE) {
    $_SESSION['media_captions_access'][] = $item['fid'];
  }

  $element['#attributes']['id'] = $element['#id'] .'-ahah-wrapper';
  $element['#prefix'] = '<div '. drupal_attributes($element['#attributes']) .'>';
  $element['#suffix'] = '</div>';

  return $element;
}

/**
 * An #upload_validators callback. Add the field to the file object.
 *
 * This validation function adds the field to the file object for later
 * use in field aware modules implementing hook_file. It's not truly a
 * validation at all, rather a convient way to add properties to the uploaded
 * file.
 */
function media_captions_validate_associate_field(&$file, $field) {
  $file->field = $field;
  return array();
}

/**
 * Delete a field file and its database record.
 *
 * @param $path
 *   A file object.
 * @param $force
 *   Force File Deletion ignoring reference counting.
 * @return mixed
 *   TRUE for success, Array for reference count block, or FALSE in the event of an error.
 */
function media_captions_delete_file($file, $force = FALSE) {
  $file = (object)$file;
  // If any module returns a value from the reference hook, the
  // file will not be deleted from Drupal, but file_delete will
  // return a populated array that tests as TRUE.
  if (!$force && $references = module_invoke_all('file_references', $file)) {
    $references = array_filter($references); // only keep positive values
    if (!empty($references)) {
      return $references;
    }
  }

  // Let other modules clean up on delete.
  module_invoke_all('file_delete', $file);

  // Make sure the file is deleted before removing its row from the
  // database, so UIs can still find the file in the database.
  if (file_delete($file->filepath)) {
    db_query('DELETE FROM {files} WHERE fid = %d', $file->fid);
    _field_file_cache(NULL, $file); // delete the file from the cache
    return TRUE;
  }
  return FALSE;
}


/**
 * Load a file from the database.
 *
 * @param $fid
 *   A numeric file id or string containing the file path.
 * @param $reset
 *   Whether to reset the internal file_load cache.
 * @return
 *   A file array.
 */
function media_captions_load_file($fid, $reset = NULL) {
  // Reset internal cache.
  if (isset($reset)) {
    _media_captions_cache_file(NULL, TRUE);
  }

  if (empty($fid)) {
    return array('fid' => 0, 'filepath' => '', 'filename' => '', 'filemime' => '', 'filesize' => 0);
  }

  $files = _media_captions_cache_file();

  // Serve file from internal cache if available.
  if (empty($files[$fid])) {
    if (is_numeric($fid)) {
      $file = db_fetch_object(db_query('SELECT f.* FROM {files} f WHERE f.fid = %d', $fid));
    }
    else {
      $file = db_fetch_object(db_query("SELECT f.* FROM {files} f WHERE f.filepath = '%s'", $fid));
    }

    if (!$file) {
      $file = array('fid' => 0, 'filepath' => '', 'filename' => '', 'filemime' => '', 'filesize' => 0);
    }

    module_invoke_all('file_load', $file);

    // Cache the fully loaded file for later use.
    $files = _media_captions_cache_file($file);
  }

  // Cast to an array for the field storage.
  // Contrary to fields, hook_file() and core file functions expect objects.
  return isset($files[$fid]) ? (array) $files[$fid] : FALSE;
}

/**
 * Internal cache, in order to minimize database queries for loading files.
 */
function _media_captions_cache_file($file = NULL, $reset = FALSE) {
  static $files = array();

  // Reset internal cache.
  if (is_object($reset)) { // file object, uncache just that one
    unset($files[$reset->fid]);
    unset($files[$reset->filepath]);
  }
  else if ($reset) { // TRUE, delete the whole cache
    $files = array();
  }

  // Cache the file by both fid and filepath.
  // Use non-copying objects to save memory.
  if (!empty($file->fid)) {
    $files[$file->fid] = $file;
    $files[$file->filepath] = $file;
  }
  return $files;
}

/**
 *  This provides extra widget settings to emfields.
 *  A checkbox to allow captions.
 */
function _media_captions_emfield_widget_settings_extra($op, $widget) {
  switch ($op) {
    case 'form':
      $form = array();
      $form['media_captions'] = array(
        '#type' => 'fieldset',
        '#title' => t('Captions & Audio descriptions'),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
      );
      $form['media_captions']['media_captions'] = array(
        '#type' => 'checkbox',
        '#title' => t('Store captions for this field'),
        '#description' => t('If checked, then editors may upload captions for this media.'),
        '#default_value' => isset($widget['media_captions']) ? $widget['media_captions'] : media_captions_variable_get('default_captions'),
      );
      $form['media_captions']['media_captions_label'] = array(
        '#type' => 'textfield',
        '#title' => t('Captions label'),
        '#default_value' => isset($widget['media_captions_label']) ? $widget['media_captions_label'] : media_captions_variable_get('label'),
        '#description' => t('This label will be displayed when uploading captions.'),
      );
      $form['media_captions']['media_captions_description'] = array(
        '#type' => 'textarea',
        '#title' => t('Captions description'),
        '#default_value' => isset($widget['media_captions_description']) ? $widget['media_captions_description'] : media_captions_variable_get('description'),
        '#description' => t('This description will be displayed when uploading captions.'),
      );
      $form['media_captions']['ad'] = array(
        '#type' => 'fieldset',
        '#title' => t('Audio descriptions'),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
      );
      $form['media_captions']['ad']['media_captions_ad'] = array(
        '#type' => 'checkbox',
        '#title' => t('Store audio descriptions for this field'),
        '#description' => t('If checked, then editors may upload audio descriptions for this media.'),
        '#default_value' => isset($widget['media_captions_ad']) ? $widget['media_captions_ad'] : media_captions_variable_get('default_ad'),
      );
      $form['media_captions']['ad']['media_captions_ad_label'] = array(
        '#type' => 'textfield',
        '#title' => t('Audio descriptions label'),
        '#default_value' => isset($widget['media_captions_ad_label']) ? $widget['media_captions_ad_label'] : media_captions_variable_get('ad_label'),
        '#description' => t('This label will be displayed when uploading audio descriptions.'),
      );
      $form['media_captions']['ad']['media_captions_ad_description'] = array(
        '#type' => 'textarea',
        '#title' => t('Audio descriptions description'),
        '#default_value' => isset($widget['media_captions_ad_description']) ? $widget['media_captions_ad_description'] : media_captions_variable_get('ad_description'),
        '#description' => t('This description will be displayed when uploading audio descriptions.'),
      );

      // Convert the extensions list to be a human-friendly comma-separated list.
      $extensions = is_string($widget['file_extensions']) ? $widget['file_extensions'] : 'mp3';
      $extensions = implode(', ', explode(' ', $extensions));
      $form['media_captions']['ad']['file_extensions'] = array(
        '#type' => 'textfield',
        '#title' => t('Permitted upload file extensions'),
        '#default_value' => $extensions,
        '#size' => 64,
        '#description' => t('Extensions a user can upload to this field. Separate extensions with a space and do not include the leading dot. Leaving this blank will allow users to upload a file with any extension.'),
        '#element_validate' => array('_filefield_widget_settings_extensions_validate'),
        '#weight' => 1,
      );

      $form['media_captions']['ad']['path_settings'] = array(
        '#type' => 'fieldset',
        '#title' => t('Path settings'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
        '#weight' => 6,
      );
      $form['media_captions']['ad']['path_settings']['file_path'] = array(
        '#type' => 'textfield',
        '#title' => t('File path'),
        '#default_value' => is_string($widget['file_path']) ? $widget['file_path'] : '',
        '#description' => t('Optional subdirectory within the "%directory" directory where files will be stored. Do not include preceding or trailing slashes.', array('%directory' => variable_get('file_directory_path', 'files') . '/')),
        '#element_validate' => array('_media_captions_widget_settings_file_path_validate'),
        '#suffix' => theme('token_help', 'user'),
      );

      $form['media_captions']['ad']['max_filesize'] = array(
        '#type' => 'fieldset',
        '#title' => t('File size restrictions'),
        '#description' => t('Limits for the size of files that a user can upload. Note that these settings only apply to newly uploaded files, whereas existing files are not affected.'),
        '#weight' => 6,
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );
      $form['media_captions']['ad']['max_filesize']['max_filesize_per_file'] = array(
        '#type' => 'textfield',
        '#title' => t('Maximum upload size per file'),
        '#default_value' => is_string($widget['max_filesize_per_file'])
                            ? $widget['max_filesize_per_file']
                            : '',
        '#description' => t('Specify the size limit that applies to each file separately. Enter a value like "512" (bytes), "80K" (kilobytes) or "50M" (megabytes) in order to restrict the allowed file size. If you leave this empty the file sizes will be limited only by PHP\'s maximum post and file upload sizes (current limit <strong>%limit</strong>).', array('%limit' => format_size(file_upload_max_size()))),
        '#element_validate' => array('_media_captions_widget_settings_max_filesize_per_file_validate'),
      );
      $form['media_captions']['ad']['max_filesize']['max_filesize_per_node'] = array(
        '#type' => 'textfield',
        '#title' => t('Maximum upload size per node'),
        '#default_value' => is_string($widget['max_filesize_per_node'])
                            ? $widget['max_filesize_per_node']
                            : '',
        '#description' => t('Specify the total size limit for all files in field on a given node. Enter a value like "512" (bytes), "80K" (kilobytes) or "50M" (megabytes) in order to restrict the total size of a node. Leave this empty if there should be no size restriction.'),
        '#element_validate' => array('_media_captions_widget_settings_max_filesize_per_node_validate'),
      );
      return $form;
    case 'save':
      return array('media_captions', 'media_captions_label', 'media_captions_description', 'media_captions_ad', 'media_captions_ad_label', 'media_captions_ad_description', 'file_extensions', 'file_path', 'max_filesize_per_file', 'max_filesize_per_node');
  }
}

/**
 *  Implements hook_emfield_field_extra().
 *  This is called on field operations to allow us to add captions.
 */
function _media_captions_emfield_field_extra($op, &$node, $field, &$items, $teaser, $page, $module) {
  switch ($op) {
    case 'insert':
    case 'update':
      // Called before content.module defaults.
      foreach ($items as $delta => $item) {
        $items[$delta]['data']['media_captions'] = array('raw' => $items[$delta]['media_captions']['media_captions_section']['media_captions'], 'frames' => media_captions_parse_data($items[$delta]['media_captions']['media_captions_section']['media_captions']));
      }
      break;
  }
}

/**
 * Menu callback; Shared AHAH callback for uploads and deletions.
 *
 * This rebuilds the form element for a particular field item. As long as the
 * form processing is properly encapsulated in the widget element the form
 * should rebuild correctly using FAPI without the need for additional callbacks
 * or processing.
 */
function media_captions_ad_widget_js($type_name, $field_name, $delta) {
  $field = content_fields($field_name, $type_name);

  if (empty($field) || empty($_POST['form_build_id'])) {
    // Invalid request.
    drupal_set_message(t('An unrecoverable error occurred. The uploaded file likely exceeded the maximum file size (@size) that this server supports.', array('@size' => format_size(file_upload_max_size()))), 'error');
    print drupal_to_js(array('data' => theme('status_messages')));
    exit;
  }

  // Build the new form.
  $form_state = array('submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  $form = form_get_cache($form_build_id, $form_state);

  if (!$form) {
    // Invalid form_build_id.
    drupal_set_message(t('An unrecoverable error occurred. This form was missing from the server cache. Try reloading the page and submitting again.'), 'error');
    print drupal_to_js(array('data' => theme('status_messages')));
    exit;
  }

  // Build the form. This calls the file field's #value_callback function and
  // saves the uploaded file. Since this form is already marked as cached
  // (the #cache property is TRUE), the cache is updated automatically and we
  // don't need to call form_set_cache().
  $args = $form['#parameters'];
  $form_id = array_shift($args);
  $form['#post'] = $_POST;
  $form = form_builder($form_id, $form, $form_state);

  $output = drupal_render($form_element);

  // AHAH is not being nice to us and doesn't know the "other" button (that is,
  // either "Upload" or "Delete") yet. Which in turn causes it not to attach
  // AHAH behaviours after replacing the element. So we need to tell it first.

  // Loop through the JS settings and find the settings needed for our buttons.
  $javascript = drupal_add_js(NULL, NULL);
  $media_captions_ahah_settings = array();
  if (isset($javascript['setting'])) {
    foreach ($javascript['setting'] as $settings) {
      if (isset($settings['ahah'])) {
        foreach ($settings['ahah'] as $id => $ahah_settings) {
          if (strpos($id, 'media-captions-upload') || strpos($id, 'media-captions-remove')) {
            $media_captions_ahah_settings[$id] = $ahah_settings;
          }
        }
      }
    }
  }

  // Add the AHAH settings needed for our new buttons.
  if (!empty($media_captions_ahah_settings)) {
    $output .= '<script type="text/javascript">jQuery.extend(Drupal.settings.ahah, '. drupal_to_js($media_captions_ahah_settings) .');</script>';
  }

  $output = theme('status_messages') . $output;

  // For some reason, file uploads don't like drupal_json() with its manual
  // setting of the text/javascript HTTP header. So use this one instead.
  $GLOBALS['devel_shutdown'] = FALSE;
  print drupal_to_js(array('status' => TRUE, 'data' => $output));
  exit;
}

